/*
CountryListing

Hello, I'm a general purpose object.
I have a collection of countries which I can add to, search, remove, etc.

I would be useful in any class that has a collection of countries
such as a Continent, TravelGuide, MarketingSystem, etc.
*/

import java.util.*;

public class CountryListing
{
	//===========================================================
	//Instance Variables

	private List<Country> countries;

	//===========================================================
	//Constructors

	public CountryListing()
	{
		//Empty constructor
		//We'll initialize our list of countries here
		this.countries = new ArrayList<Country>();
	}

	public CountryListing(List<Country> newCountries)
	{
		this.countries = newCountries;
	}

	public CountryListing(CountryListing existingListing)
	{
		/*Copy Constructor, handy when we want a copy, e.g. usage:
			CountryListing copy = new CountryListing(myListing);
		Note that the reason we "addAll" below rather then just set "this.countries"
		directly is we want our own list (we don't want to share a list
		with the method param "existingListing" (if we shared, then when
		e.g. we did an "add", it would add to both objects*/
		this();	//initialize empty array
		//Now populate
		this.addAll(existingListing.getCountries());
	}

	//===========================================================
	//Getters

	public List<Country> getCountries() { return this.countries; }

	//===========================================================
	//Basic Instance Methods

	public int size()
	{
		return getCountries().size();
	}

	public String toString()
	{
		return "Size: " + getCountries().size();
	}

	public void print()
	{
		//Print all countries in vertical list
		println(Country.fieldLabelsFormatted());
		for (Country eachCountry: this.getCountries())
			println(eachCountry);
	}

	//===========================================================
	//Adding

	public void add(Country newCountry)
	{
		this.getCountries().add(newCountry);
	}

	public void add(String newName, int newAveTemperature,
						String newCapital, int newPopulationDensity)
	{
		this.add(new Country(newName, newAveTemperature, newCapital, newPopulationDensity));
	}

	public void addAll(List<Country> newCountries)
	{
		this.getCountries().addAll(newCountries);
	}

	//===========================================================
	//Removing

	public void remove(int index)
	{
		//Remove country at method parameter "index"
		this.getCountries().remove(index);
	}

	//===========================================================
	//Getting

	public Country countryAtIndex(int index)
	{
		//Return the Country at the method parameter "index"
		//Index is 0-based (e.g. first element is index 0)
		return this.getCountries().get(index);
	}

	public List<Country> countriesFromTo(int firstIndex, int lastIndex)
	{
		//Return the Countries from index to index2 (inclusive)
		//The List "subList" method takes parameters startIndex and lastIndex+1
		//That is -- to copy elements from index=2 to index=5,
		//we would use subList(2, 5+1) or subList(2, 6)
		//exception: If we have bad index, return empty collection
		if (firstIndex < 0 || lastIndex >= this.size())
			return new ArrayList<Country>();
		return this.getCountries().subList(firstIndex, lastIndex+1);
	}

	//===========================================================
	//Finding

	public Country countryNamed(String countryName)
	{
		//Return the matching country (or null if none found)
		for (Country eachCountry: this.getCountries())
			if (eachCountry.getName().equalsIgnoreCase(countryName))
				return eachCountry;
		//None found, so return null (meaning nothing)
		return null;
	}

	public List<Country> countriesNamed(List<String> countryNames)
	{
		//Return the countries that have names that match any of the
		//names in the method parameter "countryNames"
		List<Country> matches = new ArrayList<Country>();
		for (Country eachCountry: this.getCountries())
			if (countryNames.contains(eachCountry.getName()))
				matches.add(eachCountry);
		return matches;
	}

	public Country countryWithCapital(String capitalName)
	{
		//Return the matching country that has capital matching method param
		for (Country eachCountry: this.getCountries())
			if (eachCountry.getCapital().equalsIgnoreCase(capitalName))
				return eachCountry;
		return null;
	}

	public List<Country> selectWarmerThan(int thresholdTemperatureF)
	{
		//Return a list of all countries warmer then method param
		List<Country> matches = new ArrayList<Country>();
		for (Country eachCountry: this.getCountries())
			if (eachCountry.getAveTemperature() > thresholdTemperatureF)
				matches.add(eachCountry);
		return matches;
	}

	//===========================================================
	//Sorting

	public void sortByCountryName()
	{
		//Here we sort our contries by population density (ascending)
		//Our ivar will be sorted
		this.countries.sort((country1, country2)
								-> country1.getName().compareTo(country2.getName()));
	}

	public void sortByAveTemperature()
	{
		//Here we sort our contries by ave temperature (ascending)
		//Our ivar will be sorted
		this.countries.sort((country1, country2)
								-> Integer.compare(country1.getAveTemperature(), country2.getAveTemperature()));
	}

	public void sortByPopulationDensity()
	{
		//Here we sort our contries by population density (ascending)
		//Our ivar will be sorted
		this.countries.sort((country1, country2)
								-> Integer.compare(country1.getPopulationDensity(), country2.getPopulationDensity()));
	}

	public void sortByPopulationDensityDescending()
	{
		//Just like "sortByPopulationDensity" except we reverse the comparison (i.e. descending)
		this.countries.sort((country1, country2)
								-> Integer.compare(country2.getPopulationDensity(), country1.getPopulationDensity()));
	}

	//===========================================================
	//Test Code

	public static void main(String[] args)
	{
		CountryListing countryListing;

		countryListing = new CountryListing();
		countryListing.testSmallSet();

		println("------------------");
		countryListing = new CountryListing();
		countryListing.testLargeSet();
	}

	public void testSmallSet()
	{
		this.add("Ghana", 81, "Accra", 259);
		this.add("Mongolia", 31, "Ulaanbaatar", 4);
		this.print();
	}

	public void testLargeSet()
	{
		this.loadSampleData1();
		this.sortByAveTemperature();
		this.print();
	}

	public void loadSampleData1()
	{
		String[][] data = sampleData1();
		if (data.length == 0) return;
		for (int row=0; row<data.length; row++)
		{
			String[] rowArray = data[row];
			String nm, cap;
			int aveTemp, pop;
			nm = rowArray[0];
			aveTemp = Integer.valueOf(rowArray[1]);
			cap = rowArray[2];
			pop = Integer.valueOf(rowArray[3]);
			this.add(nm, aveTemp, cap, pop);
		}
	}

	public static String[][] sampleData1()
	{
		String[][] data =
			{
			{"Ghana", "81", "Accra", "259"},
			{"Somalia", "81", "Mogadishu", "36"},
			{"Jamaica", "77", "Kingston", "640"},
			{"Vietnam", "76", "Hanoi", "671"},
			{"Nicaragua", "77", "Managua", "114"},
			{"Iraq", "71", "Baghdad", "181"},
			{"Mexico", "70", "Mexico City", "148"},
			{"Canada", "22", "Ottawa", "9"},
			{"Norway", "35", "Oslo", "36"},
			{"Japan", "52", "Tokyo", "873"},
			{"Laos", "73", "Vientiane", "70"},
			{"India", "75", "New Delhi", "943"},
			{"North Korea", "42", "Pyongyang", "518"},
			{"South Korea", "53", "Seoul", "1261"},
			{"Mongolia", "31", "Ulaanbaatar", "4"},
			{"Iceland", "35", "Reykjavik", "8"}
			};
		return data;
	}

	public static void println(Object o) { System.out.println(o); }

}



